#!/usr/bin/env bash

# bpgallery.sh - builds a simple 'gallery' from a collection of images
# Copyright (C) 2004 Brett Parker <iDunno@sommitrealweird.co.uk>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

set -f

VERSION="1.0.3"

function bpgallery_default_head() {
cat <<END
<?xml version="1.0"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "xhtml11.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en">
<head>
	<title>${TITLE}</title>
	<link rel="stylesheet" href="style.css" type="text/css" />
</head>
<body>
END
}

declare -rf bpgallery_default_head

function bpgallery_default_description() {
cat <<END
	<h1>${TITLE}</h1>
	${DESCRIPTION}
END
}

declare -rf bpgallery_default_description

function bpgallery_default_thumbsline() {
if [[ -z $caption ]]; then
	caption_alt=Unknown
else
	caption_alt=$caption
fi
cat << END
<div class="thumbnail"><a href="$link"><img src="icons/$filename" alt="$caption_alt" /></a><div class="caption">$caption</div></div>
END
}

declare -rf bpgallery_default_thumbsline

function bpgallery_default_tail() {
cat << END
</body>
</html>
END
}

declare -rf bpgallery_default_tail

function bpgallery_default_stylesheet() {
cat <<END
body {
	background: white;
	color: black;
	font-family: sans-serif;
	font-size: 10pt;
}

div.thumbnail {
	float: left;
	padding: 20px;
	border: 0px;
	width: ${WIDTH}px;
	height: ${MAXHEIGHT}px;
}

div.caption {
	width: 100%;
	text-align: center;
	font-weight: bold;
}

div.navigation {
	width: 100%;
	margin: 2em;
	text-align: center;
}

div.navigation ul {
	width: 100%;
	display: inline;
	text-align: center;
}

div.navigation ul li {
	display: inline;
	margin: 2em;
}

a {
	border: 0px;
}

img {
	border: 0px;
}
END
}

declare -rf bpgallery_default_stylesheet

function bpgallery_default_page() {
	$BPGALLERY_HEAD_FUNCTION
	cat <<END
<h1>${caption}</h1>
<div class="navigation">
	<ul>
		<li><a href='index.html'>Thumbnails</a></li>
END
	if [[ ! -z $previouspage ]]; then
		echo "<li><a href='${previouspage}'>Previous</a></li>"
	else
		echo "<li>Previous</li>"
	fi

	if [[ ! -z $nextpage ]]; then
		echo "<li><a href='${nextpage}'>Next</a></li>"
	else
		echo "<li>Next</li>"
	fi
cat <<END
	</ul>
</div>
<div class="mainimage">
	<img src="${filename}" alt="${caption}" />
</div>
END
	$BPGALLERY_TAIL_FUNCTION
}

declare -rf bpgallery_default_page

function bpgallery_escape_url() {
	temp=$1
	temp=${temp//\%/%25}
	temp=${temp//:/%3A}
	temp=${temp//;/%3B}
	temp=${temp// /%20}
	temp=${temp//$/%24}
	temp=${temp//&/%26}
	temp=${temp//+/%2B}
	temp=${temp//,/%2C}
	temp=${temp//\//%2F}
	temp=${temp//=/%3D}
	temp=${temp//\?/%3F}
	temp=${temp//@/%40}
	temp=${temp//\"/%22}
	$ECHOCOMMAND $temp
}

declare -rf bpgallery_escape_url

if [[ ! -z ${BPGALLERY_THEME} ]] ; then
	declare -r BPGALLERY_THEME
fi

if [[ ! -z ${TITLE} ]] ; then
	declare -r TITLE
fi

if [[ -e /usr/local/etc/bpgallery/config ]] ; then
	. /usr/local/etc/bpgallery/config 2>/dev/null
fi

if [[ -e /etc/bpgallery/config ]] ; then
	. /etc/bpgallery/config 2>/dev/null
fi

if [[ -e $HOME/.bpgallery.rc ]]; then
	. $HOME/.bpgallery.rc 2>/dev/null
fi

if [[ -e $1 && -d $1 && -e $1/.bpgallery.rc ]]; then
	. $1/.bpgallery.rc 2>/dev/null
fi

if [[ -z ${BPGALLERY_THEME} ]]; then
	BPGALLERY_THEME=default
fi

if [[ ! -z ${BPGALLERY_THEME_DIR} ]] && \
	[[ -e ${BPGALLERY_THEME_DIR}/${BPGALLERY_THEME} ]]; then
	. ${BPGALLERY_THEME_DIR}/${BPGALLERY_THEME} 2>/dev/null
elif [[ -e $HOME/.bpgallery.themes/${BPGALLERY_THEME} ]]; then
	. $HOME/.bpgallery.themes/${BPGALLERY_THEME} 2>/dev/null
elif [[ -e /usr/local/etc/bpgallery/themes/${BPGALLERY_THEME} ]]; then
	. /usr/local/etc/bpgallery/themes/${BPGALLERY_THEME} 2>/dev/null
elif [[ -e /etc/bpgallery/themes/${BPGALLERY_THEME} ]]; then
	. /etc/bpgallery/themes/${BPGALLERY_THEME} 2>/dev/null
fi

if [[ -z $TITLE ]]; then 
	TITLE="Photo Gallery"
fi

if [[ -z $CONVERTTOOL ]]; then
	CONVERTTOOL=convert
fi

if [[ -z $FINDCOMMAND ]]; then
	FINDCOMMAND=find
fi

if [[ -z $XARGSCOMMAND ]]; then
	XARGSCOMMAND=xargs
fi

if [[ -z $ECHOCOMMAND ]]; then
	ECHOCOMMAND=echo
fi

if [[ -z $SORTCOMMAND ]]; then
	SORTCOMMAND=sort
fi

if [[ -z $IDENTIFYCOMMAND ]]; then
	IDENTIFYCOMMAND=identify
fi

if [[ -z $HEADCOMMAND ]]; then
	HEADCOMMAND=head
fi

if [[ -z $GREPCOMMAND ]]; then
	GREPCOMMAND=grep
fi

if [[ -z $SEDCOMMAND ]]; then
	SEDCOMMAND=sed
fi

if [[ -z $WIDTH ]]; then
	WIDTH=100
fi

if [[ -z $IMAGEEXTENSIONS ]]; then
	IMAGEEXTENSIONS="jpeg jpg gif png";
fi

if [[ -z $WCCOMMAND ]]; then
	WCCOMMAND="wc -l"
fi

if [[ -z $CAPTIONHEIGHT ]]; then
	CAPTIONHEIGHT=75
fi

if [[ -z $OUTPUTHTML ]]; then
	OUTPUTHTML=0
fi

if [[ -z $GENERATEPAGESFORFULLSIZE ]]; then
	GENERATEPAGESFORFULLSIZE=0
fi

if [[ $GENERATEPAGESFORFULLSIZE != 0 ]]; then
	if declare -F "bpgallery_${BPGALLERY_THEME}_page" > /dev/null ; then
		BPGALLERY_PAGE_FUNCTION="bpgallery_${BPGALLERY_THEME}_page"
	else
		BPGALLERY_PAGE_FUNCTION="bpgallery_default_page"
	fi
fi

if declare -F "bpgallery_${BPGALLERY_THEME}_head" > /dev/null ; then
	BPGALLERY_HEAD_FUNCTION="bpgallery_${BPGALLERY_THEME}_head"
else
	BPGALLERY_HEAD_FUNCTION="bpgallery_default_head"
fi

if declare -F "bpgallery_${BPGALLERY_THEME}_description" > /dev/null ; then
	BPGALLERY_DESCRIPTION_FUNCTION="bpgallery_${BPGALLERY_THEME}_description"
else
	BPGALLERY_DESCRIPTION_FUNCTION="bpgallery_default_description"
fi

if declare -F "bpgallery_${BPGALLERY_THEME}_thumbsline" > /dev/null ; then
	BPGALLERY_THUMBSLINE_FUNCTION="bpgallery_${BPGALLERY_THEME}_thumbsline"
else
	BPGALLERY_THUMBSLINE_FUNCTION="bpgallery_default_thumbsline"
fi

if declare -F "bpgallery_${BPGALLERY_THEME}_tail" > /dev/null ; then
	BPGALLERY_TAIL_FUNCTION="bpgallery_${BPGALLERY_THEME}_tail"
else
	BPGALLERY_TAIL_FUNCTION="bpgallery_default_tail"
fi

if declare -F "bpgallery_${BPGALLERY_THEME}_stylesheet" > /dev/null ; then
	BPGALLERY_STYLESHEET_FUNCTION="bpgallery_${BPGALLERY_THEME}_stylesheet"
else
	BPGALLERY_STYLESHEET_FUNCTION="bpgallery_default_stylesheet"
fi

FINDIMAGESOPTIONS=""

for imageext in $IMAGEEXTENSIONS; do
	FINDIMAGESOPTIONS=$FINDIMAGESOPTIONS' -o -type f -iname '*.$imageext' -print0'
done

FINDIMAGESOPTIONS=${FINDIMAGESOPTIONS## -o }
FINDIMAGESOPTIONS='-maxdepth 1 '${FINDIMAGESOPTIONS}

function usage() {
cat <<END
Usage:
  $0 [--help|--version|<path to images>]

    --help
        displays this help screen
    --version
        displays the version and exits
	
END
}

function version() {
cat <<END
Version: $VERSION
END
}

function generate_thumbs() {
	$ECHOCOMMAND "Generating Thumbnails"
	currentimage=0
	totalimages=$($FINDCOMMAND . $FINDIMAGESOPTIONS | $XARGSCOMMAND -0 --replace $ECHOCOMMAND {} | $WCCOMMAND);
	imagestoupdate=$($FINDCOMMAND . $FINDIMAGESOPTIONS | $XARGSCOMMAND -0 --replace $ECHOCOMMAND {} | while read filename; do filename=${filename//\"/\\\"}; if [ ! -r "icons/$filename" ] || [ "$filename" -nt "icons/$filename" ] || [ $($IDENTIFYCOMMAND -format "%w" "icons/$filename") -ne $WIDTH ]; then $ECHOCOMMAND $filename; fi; done | $WCCOMMAND)
	
	if [ $totalimages -eq $imagestoupdate ]; then
		$ECHOCOMMAND "Regenerating all thumbnails"
	elif [ $imagestoupdate -eq 0 ]; then
		$ECHOCOMMAND "No Updated Thumbs, not regenerating"
		return
	else
		$ECHOCOMMAND "Generating $imagestoupdate of $totalimages thumbnails"
	fi
	
	$FINDCOMMAND . $FINDIMAGESOPTIONS | $XARGSCOMMAND -0 --replace $ECHOCOMMAND {} | while read filename; do tempfilename=${filename//\"/\\\"/}; if [ ! -r "icons/$tempfilename" ] || [ "$tempfilename" -nt "icons/$tempfilename" ] || [ $($IDENTIFYCOMMAND -format "%w" "icons/$filename") -ne $WIDTH ]; then $ECHOCOMMAND -n $filename; $ECHOCOMMAND -n -e "\000"; fi; done | $XARGSCOMMAND -0 --verbose --max-procs=4 --replace $CONVERTTOOL -resize $WIDTH '{}' 'icons/{}' 2>&1 | while read throwout; do $ECHOCOMMAND done: $currentimage/$imagestoupdate images; currentimage=$((currentimage+1)); done
	$ECHOCOMMAND done: $imagestoupdate/$imagestoupdate images
	$ECHOCOMMAND "Completed generating thumbs for $totalimages images"
}

function generate_pages() {
	$ECHOCOMMAND "Generating Pages"
	previouspage=""
	currentpage=""
	nextpage=""
	previousimage=""
	currentimage=""
	nextimage=""

	$FINDCOMMAND . $FINDIMAGESOPTIONS | \
	$XARGSCOMMAND -0 --replace $ECHOCOMMAND {} | \
	$SORTCOMMAND -g | \
	while read imagefilename; do
		previousimage=$currentimage
		currentimage=$nextimage
		nextimage=$imagefilename
		previouspage=$currentpage
		currentpage=$nextpage
		nextpage=${imagefilename}.html
		filename=${currentimage}
		generate_general_page "$previouspage" "$currentpage" "$nextpage"
		echo $nextimage
	done | tail -n 2 | (
		read previouspage
		read currentpage
		filename=${currentpage}
		currentpage=${currentpage}.html
		previouspage=${previouspage}.html
		nextpage=""
		generate_general_page "$previouspage" "$currentpage" ""
	)
}



function generate_general_page() {

	if [[ -z $1 ]]; then
		previouspage=""
	fi

	if [[ -z $2 ]]; then
		currentpage=""
		return
	fi

	if [[ -z $3 ]]; then
		nextpage=""
	fi

	if [ -r captions.txt ]; then
		caption=$($GREPCOMMAND -E "^${filename}   " captions.txt); caption=${caption#*    }
	else
		caption=""
	fi

	$BPGALLERY_PAGE_FUNCTION > "$currentpage"
}

if [[ $OUTPUTHTML != 0 ]]; then
	$ECHOCOMMAND "<pre>"
fi

if [[ -z $1 ]]; then
	$ECHOCOMMAND "No path given"
	usage
	exit 1
fi

if [[ "$1" == "--help" || "$1" == "-h" ]]; then
	usage
	exit 0
fi

if [[ "$1" == "--version" || "$1" == "-v" ]]; then
	version
	exit 0
fi

if [[ ! -d $1 ]]; then
	$ECHOCOMMAND "$1 is not a directory"
	usage
	exit 2
fi

cd "$1"

if ( ! $FINDCOMMAND . $FINDIMAGESOPTIONS > /dev/null 2>/dev/null ); then
	$ECHOCOMMAND "$1 does not contain any images. Quitting."
	exit 4
fi

if [ ! -w . ]; then
	$ECHOCOMMAND "Can't write to images directory, exiting"
	exit 8
fi

if [ -e index.html ] && [ ! -w index.html ]; then
	$ECHOCOMMAND "Can't write index.html, exiting"
	exit 8
fi

if [ -e style.css ] && [ ! -w style.css ]; then
	$ECHOCOMMAND "Can't write style.css, exiting"
	exit 8
fi

if [ ! -d icons ]; then
	mkdir icons
fi

if [ ! -w icons ]; then
	$ECHOCOMMAND "Can't write to icons directory, exiting"
	exit 16
fi

generate_thumbs

if [ $GENERATEPAGESFORFULLSIZE != 0 ]; then
	generate_pages
fi

if [ -r description.txt ] ; then
	DESCRIPTION=$($SEDCOMMAND -e '1 { s/^/<p>/; }; /^$/ { s,$,</p><p>,; }; $ { s,$,</p>, };' description.txt)
else
	DESCRIPTION=""
fi


$ECHOCOMMAND "Starting to generate page"

$BPGALLERY_HEAD_FUNCTION > index.html
$BPGALLERY_DESCRIPTION_FUNCTION >> index.html

$ECHOCOMMAND "Adding Captions"

$FINDCOMMAND . $FINDIMAGESOPTIONS | $XARGSCOMMAND -0 --replace $ECHOCOMMAND {} |$SORTCOMMAND -g | while read filename; do filename=${filename#./}; if [ -r captions.txt ]; then caption=$($GREPCOMMAND -E "^$filename	" captions.txt); caption=${caption#*	}; else caption=""; fi; if [ $GENERATEPAGESFORFULLSIZE != 0 ]; then link=$(bpgallery_escape_url "${filename}.html"); else link=$(bpgallery_escape_url "$filename"); fi; filename=$(bpgallery_escape_url "$filename"); $BPGALLERY_THUMBSLINE_FUNCTION; done >> index.html

$BPGALLERY_TAIL_FUNCTION >> index.html

$ECHOCOMMAND "Finished generating the page"
$ECHOCOMMAND "Generating stylesheet"
cd icons

MAXHEIGHT=$($FINDCOMMAND . $FINDIMAGESOPTIONS | $XARGSCOMMAND -0 $IDENTIFYCOMMAND -format "%h\n" | $GREPCOMMAND -v "^$" | $SORTCOMMAND -g -r | $HEADCOMMAND -n 1)
MAXWIDTH=$($FINDCOMMAND . $FINDIMAGESOPTIONS | $XARGSCOMMAND -0 $IDENTIFYCOMMAND -format "%w\n" | $GREPCOMMAND -v "^$" | $SORTCOMMAND -g -r | $HEADCOMMAND -n 1)

cd ..

# add a bit to the maxheight for the size of the caption
MAXHEIGHT=$((MAXHEIGHT+$CAPTIONHEIGHT))

$BPGALLERY_STYLESHEET_FUNCTION > style.css

$ECHOCOMMAND "All done"

if [[ $OUTPUTHTML != 0 ]]; then
	$ECHOCOMMAND "</pre>"
fi
